/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.adl.compiler;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import org.objectweb.fractal.adl.components.Component;
import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.adl.components.PrimitiveCompiler;
import org.objectweb.fractal.adl.implementations.Implementation;
import org.objectweb.fractal.adl.implementations.ImplementationContainer;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.cecilia.adl.directives.Cflag;
import org.objectweb.fractal.cecilia.adl.directives.CflagContainer;
import org.objectweb.fractal.cecilia.adl.source.SourceFileProvider;
import org.objectweb.fractal.cecilia.adl.types.TypeDecorationUtil;
import org.objectweb.fractal.task.codegen.api.FileProviderTask;
import org.objectweb.fractal.task.codegen.api.RequireFileCollectionProviderTask;
import org.objectweb.fractal.task.codegen.api.RequireSourceFileProviderTask;
import org.objectweb.fractal.task.codegen.api.SourceFileProviderTask;
import org.objectweb.fractal.task.codegen.util.SourceFile;
import org.objectweb.fractal.task.core.AbstractTask;
import org.objectweb.fractal.task.core.TaskMap;
import org.objectweb.fractal.task.core.TaskMap.TaskHole;

/**
 * Abstract output compiler.
 */
public abstract class AbstractOutputCompiler
    implements
      PrimitiveCompiler,
      BindingController {

  // --------------------------------------------------------------------------
  // Client interfaces.
  // --------------------------------------------------------------------------

  /**
   * The Name of the {@link CompilerBuilder} client interface used by this
   * compiler.
   * 
   * @see #compilationBuilderItf
   */
  public static final String COMPILER_BUILDER_ITF     = "compiler-builder";

  /**
   * The Name of the {@link SourceFileProvider} client interface used by this
   * compiler.
   * 
   * @see #sourceFileProviderItf
   */
  public static final String SOURCE_FILE_PROVIDER_ITF = "source-file-provider";

  /** The {@link CompilerBuilder} used by this compiler. */
  public CompilerBuilder     compilationBuilderItf;

  /**
   * The {@link SourceFileProvider} used by this compiler to find the set of
   * {@link SourceFileProviderTask} that provide sources to be compiled for each
   * components.
   */
  public SourceFileProvider  sourceFileProviderItf;

  // --------------------------------------------------------------------------
  // Utility methods
  // --------------------------------------------------------------------------

  public void bindFc(final String itf, final Object value)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException {

    if (itf == null) {
      throw new IllegalArgumentException("Interface name can't be null");
    }

    if (itf.equals(COMPILER_BUILDER_ITF)) {
      this.compilationBuilderItf = (CompilerBuilder) value;
    } else if (itf.equals(SOURCE_FILE_PROVIDER_ITF)) {
      this.sourceFileProviderItf = (SourceFileProvider) value;
    } else {
      throw new NoSuchInterfaceException("There is no interface named '" + itf
          + "'");
    }

  }

  public String[] listFc() {
    return new String[]{COMPILER_BUILDER_ITF, SOURCE_FILE_PROVIDER_ITF};
  }

  public Object lookupFc(final String itf) throws NoSuchInterfaceException {
    if (itf == null) {
      throw new IllegalArgumentException("Interface name can't be null");
    }

    if (itf.equals(COMPILER_BUILDER_ITF)) {
      return this.compilationBuilderItf;
    } else if (itf.equals(SOURCE_FILE_PROVIDER_ITF)) {
      return this.sourceFileProviderItf;
    } else {
      throw new NoSuchInterfaceException("There is no interface named '" + itf
          + "'");
    }
  }

  public void unbindFc(final String itf) throws NoSuchInterfaceException,
      IllegalBindingException, IllegalLifeCycleException {

    if (itf == null) {
      throw new IllegalArgumentException("Interface name can't be null");
    }

    if (itf.equals(COMPILER_BUILDER_ITF)) {
      this.compilationBuilderItf = null;
    } else if (itf.equals(SOURCE_FILE_PROVIDER_ITF)) {
      this.sourceFileProviderItf = null;
    } else {
      throw new NoSuchInterfaceException("There is no interface named '" + itf
          + "'");
    }
  }

  protected Collection<TaskMap.TaskHole> getComponentCompilationTasks(
      final ComponentContainer container, final TaskMap tasks,
      final Map<Object, Object> context) {

    final List<TaskMap.TaskHole> compilationTasks = new ArrayList<TaskMap.TaskHole>();

    final StringBuffer cFlagBuffer = new StringBuffer();

    Implementation impl = null;
    if (container instanceof ImplementationContainer)
      impl = ((ImplementationContainer) container).getImplementation();
    if (impl != null && impl instanceof CflagContainer) {
      for (final Cflag cflag : ((CflagContainer) impl).getCflags()) {
        cFlagBuffer.append(cflag.getValue()).append(' ');
      }
    }
    final String cFlag = cFlagBuffer.toString();

    // retrieve the set of SourceFileProviderTask that provides source file to
    // be compiled
    final Collection<TaskMap.TaskHole> sourceFileTasks = sourceFileProviderItf
        .getSourceFileProviderTask(container, tasks, context);
    for (final TaskMap.TaskHole sourceFileProviderTaskHole : sourceFileTasks) {

      // get the compilation task for of the source file provided by
      // sourceFileProviderTaskHole
      final TaskMap.TaskHole compilationTaskHole = tasks.getTaskHole(
          "compilation", sourceFileProviderTaskHole);

      // if the compilation task does not exist
      if (!compilationTaskHole.isFilled()) {
        // create compilation task
        final CompilationTask compilationTask = new CompilationTask(
            compilationBuilderItf, cFlag);
        compilationTask.setSourceFileProviderTask(sourceFileProviderTaskHole);
        tasks.addTask("compilation", sourceFileProviderTaskHole,
            compilationTask);
      }
      compilationTasks.add(compilationTaskHole);
    }

    return compilationTasks;
  }

  protected void addSubCompDependancies(final ComponentContainer container,
      final TaskMap tasks, final RequireFileCollectionProviderTask outputTask) {
    for (final Component subComponent : container.getComponents()) {
      outputTask.addFileCollectionProviderTask(tasks.getTaskHole("output",
          TypeDecorationUtil.getTypeDecoration(subComponent)));
    }
  }

  // --------------------------------------------------------------------------
  // Task classes
  // --------------------------------------------------------------------------

  protected static class CompilationTask extends AbstractTask
      implements
        FileProviderTask,
        RequireSourceFileProviderTask {

    private final CompilerBuilder builder;
    private final String          cFlags;
    private TaskMap.TaskHole      sourceFileProviderTask;
    private File                  result;

    /**
     * @param builder
     * @param cFlags
     */
    public CompilationTask(final CompilerBuilder builder, final String cFlags) {
      this.builder = builder;
      this.cFlags = cFlags;
    }

    // ------------------------------------------------------------------------
    // Implementation of the FileProviderTask interface
    // ------------------------------------------------------------------------

    public File getFile() {
      return result;
    }

    // ------------------------------------------------------------------------
    // Implementation of the RequireSourceFileProviderTask interface
    // ------------------------------------------------------------------------

    public SourceFileProviderTask getSourceFileProviderTask() {
      return (SourceFileProviderTask) sourceFileProviderTask.getTask();
    }

    public void setSourceFileProviderTask(final TaskHole task) {
      sourceFileProviderTask = task;
      addPreviousTask(task);
    }

    // ------------------------------------------------------------------------
    // Implementation of the Task interface
    // ------------------------------------------------------------------------

    public Object getResult() {
      return getFile();
    }

    public void execute(final Map<Object, Object> context) throws Exception {
      final SourceFile sourceFile = getSourceFileProviderTask().getSourceFile();

      // Prepare the destination file
      final File destDir = (File) context.get("objBuildDirectory");
      final String fileName = sourceFile.getSignature().replace('.',
          File.separatorChar)
          + ".o";
      result = new File(destDir, fileName);
      builder.compile(sourceFile.getFile(), result, cFlags, context);
    }

    // ------------------------------------------------------------------------
    // Overridden Task methods
    // ------------------------------------------------------------------------

    @Override
    protected boolean doAddDependency(final TaskHole task, final Class<?> role,
        final Object context) {
      if (super.doAddDependency(task, role, context)) return true;

      if (role == RequireSourceFileProviderTask.SOURCE_FILE_PROVIDER_TASK_ROLE) {
        setSourceFileProviderTask(task);
        return true;
      }
      return false;
    }

    @Override
    protected boolean doRemoveDependency(final TaskHole task,
        final Class<?> role) {
      if (super.doRemoveDependency(task, role)) return true;

      if (role == RequireSourceFileProviderTask.SOURCE_FILE_PROVIDER_TASK_ROLE) {
        setSourceFileProviderTask(null);
        return true;
      }
      return false;
    }

  }

  protected static final class IncludeKey {

    protected final String signature;
    protected final String flags;

    /**
     * @param signature
     * @param flags
     */
    public IncludeKey(final String signature, final String flags) {
      this.signature = signature;
      this.flags = flags;
    }

    @Override
    public boolean equals(final Object key) {
      if (!(key instanceof IncludeKey) || key == null) return false;
      final IncludeKey candidate = (IncludeKey) key;

      return ((candidate.signature == null)
          ? signature == null
          : candidate.signature.equals(signature))
          && ((candidate.flags == null) ? flags == null : candidate.flags
              .equals(flags));
    }

    @Override
    public int hashCode() {
      return ((signature == null) ? 0 : signature.hashCode())
          + ((flags == null) ? 0 : flags.hashCode());
    }

    @Override
    public String toString() {
      return "{" + signature + ", " + flags + "}";
    }

  }

}
