/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Ali Erdem Ozcan
 */

package org.objectweb.fractal.cecilia.adl.implementations;

import static org.objectweb.fractal.cecilia.adl.implementations.ImplementationDecorationUtil.getCode;

import java.util.List;
import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Node;
import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.adl.implementations.Implementation;
import org.objectweb.fractal.adl.implementations.ImplementationContainer;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.cecilia.adl.components.AbstractPrimitiveCompiler;
import org.objectweb.fractal.cecilia.adl.directives.Include;
import org.objectweb.fractal.cecilia.adl.directives.IncludeContainer;
import org.objectweb.fractal.task.codegen.lib.AbstractRecursiveSourceCodeProviderTask;
import org.objectweb.fractal.task.codegen.util.SourceFile;
import org.objectweb.fractal.task.core.TaskMap;
import org.objectweb.fractal.task.core.TaskMap.TaskHole;

/**
 * Adds a task for the definition of the implementation of a component.
 */
public class ImplementationCompiler extends AbstractPrimitiveCompiler
    implements
      BindingController {

  // --------------------------------------------------------------------------
  // Client interfaces
  // --------------------------------------------------------------------------

  /**
   * The name of the {@link ImplementationBuilder} client interface used by this
   * compiler.
   */
  public static final String   BUILDER_ITF        = "builder";

  /**
   * The name of the {@link ImplementationBuilder} client interface used by this
   * compiler for multi-module components.
   */
  public static final String   MODULE_BUILDER_ITF = "module-builder";

  /** The {@link ImplementationBuilder} used by this compiler. */
  public ImplementationBuilder builderItf;

  /**
   * The {@link ImplementationBuilder} used by this compiler for multi-module
   * components.
   */
  public ImplementationBuilder moduleBuilderItf;

  // --------------------------------------------------------------------------
  // Implementation of the PrimitiveCompiler interface
  // --------------------------------------------------------------------------

  public void compile(final List<ComponentContainer> path,
      final ComponentContainer container, final TaskMap tasks,
      final Map<Object, Object> context) throws ADLException {
    Implementation impl = null;
    if (container instanceof ImplementationContainer) {
      impl = ((ImplementationContainer) container).getImplementation();
    }
    // if this component does not contains implementation, return.
    if (impl == null) return;

    final DefineImplementationTask task = new DefineImplementationTask(
        builderItf, getCode((Node) impl), null);

    addTask("implementation", task, container, tasks);

    if (impl instanceof IncludeContainer) {
      final TaskHole typeTask = tasks.getTaskHole("type", container);
      for (final Include include : ((IncludeContainer) impl).getIncludes()) {
        // retrieve source file associated to this include
        final SourceFile code = (SourceFile) getCode((Node) include);

        // if source file is an assembly file, do not create module
                // task.
        if (code.isAssemblyFile()) continue;

        // create an implementation task for this include
        final DefineImplementationTask moduleTask = new DefineImplementationTask(
            moduleBuilderItf, code, null);
        moduleTask.setFactoryProviderTask(typeTask);
        tasks.addTask("module", include, moduleTask);
      }
    }
  }

  // --------------------------------------------------------------------------
  // Task class
  // --------------------------------------------------------------------------

  protected static class DefineImplementationTask
      extends
        AbstractRecursiveSourceCodeProviderTask {
    private final ImplementationBuilder builder;
    private final Object                contentDesc;
    private final Object                controllerDesc;

    /**
     * Constructor.
     * 
     * @param builder the builder.
     * @param contentDesc the description of the content part of the component.
     * @param controllerDesc the description of the controller part of the
     *            component.
     */
    public DefineImplementationTask(final ImplementationBuilder builder,
        final Object contentDesc, final Object controllerDesc) {
      this.builder = builder;
      this.contentDesc = contentDesc;
      this.controllerDesc = controllerDesc;
    }

    public void execute(final Map<Object, Object> context) throws Exception {
      final Object factory = (getFactoryProviderTask() == null)
          ? null
          : getFactoryProviderTask().getFactory();
      setResult(builder.defineImplementation(factory, contentDesc,
          controllerDesc, getCodePieces(), context));
    }
  }

  // --------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // --------------------------------------------------------------------------

  public void bindFc(final String itf, final Object value)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException {

    if (itf == null) {
      throw new IllegalArgumentException("Interface name can't be null");
    }

    if (itf.equals(BUILDER_ITF)) {
      this.builderItf = (ImplementationBuilder) value;
    } else if (itf.equals(MODULE_BUILDER_ITF)) {
      this.moduleBuilderItf = (ImplementationBuilder) value;
    } else {
      throw new NoSuchInterfaceException("There is no interface named '" + itf
          + "'");
    }
  }

  public String[] listFc() {
    return new String[]{BUILDER_ITF, MODULE_BUILDER_ITF};
  }

  public Object lookupFc(final String itf) throws NoSuchInterfaceException {

    if (itf == null) {
      throw new IllegalArgumentException("Interface name can't be null");
    }

    if (itf.equals(BUILDER_ITF)) {
      return this.builderItf;
    } else if (itf.equals(MODULE_BUILDER_ITF)) {
      return this.moduleBuilderItf;
    } else {
      throw new NoSuchInterfaceException("There is no interface named '" + itf
          + "'");
    }
  }

  public void unbindFc(final String itf) throws NoSuchInterfaceException,
      IllegalBindingException, IllegalLifeCycleException {
    if (itf == null) {
      throw new IllegalArgumentException("Interface name can't be null");
    }

    if (itf.equals(BUILDER_ITF)) {
      this.builderItf = null;
    } else if (itf.equals(MODULE_BUILDER_ITF)) {
      this.moduleBuilderItf = null;
    } else {
      throw new NoSuchInterfaceException("There is no interface named '" + itf
          + "'");
    }
  }
}
