/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.adl.source;

import java.util.Map;

import org.objectweb.fractal.adl.components.PrimitiveCompiler;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.task.codegen.api.SourceFileProviderTask;
import org.objectweb.fractal.task.codegen.lib.AbstractRecursiveSourceCodeProviderTask;
import org.objectweb.fractal.task.codegen.lib.AbstractSourceFileProviderTask;
import org.objectweb.fractal.task.codegen.util.SourceFile;
import org.objectweb.fractal.task.core.AbstractTask;

/**
 * A <code>SourceFileCompiler</code> is responsible to add the
 * {@link SourceFileProviderTask} in the task map for each components.
 */
public abstract class AbstractSourceFileCompiler
    implements
      PrimitiveCompiler,
      BindingController {

  // --------------------------------------------------------------------------
  // Client interface
  // --------------------------------------------------------------------------

  /** the name of the client builder interface. */
  public static final String BUILDER_ITF = "builder";

  /** The {@link SourceFileBuilder} used by this compiler. */
  public SourceFileBuilder   builderItf;

  // --------------------------------------------------------------------------
  // Task classes
  // --------------------------------------------------------------------------

  /**
   * This task execute the {@link SourceFileBuilder}.
   */
  protected static class SourceFileTask extends AbstractSourceFileProviderTask {

    protected final SourceFileBuilder builder;

    /**
     * @param builder
     */
    public SourceFileTask(final SourceFileBuilder builder) {
      super(AbstractRecursiveSourceCodeProviderTask.INSERTION_ORDER);
      this.builder = builder;
    }

    public void execute(final Map<Object, Object> context) throws Exception {
      final Object factory = (getFactoryProviderTask() == null)
          ? null
          : getFactoryProviderTask().getFactory();
      final Object instance = (getInstanceProviderTask() == null)
          ? null
          : getInstanceProviderTask().getInstance();
      setResult(builder.createSourceFile(factory, instance, getCodePieces(),
          context));
    }
  }

  /**
   * This task does nothing unless returning a given SourceFile object. This is
   * useful when a task requires a {@link SourceFile} that is not generated
   * dynamically.
   */
  public static final class BasicSourceFileProviderTask extends AbstractTask
      implements
        SourceFileProviderTask {

    private final SourceFile sourceFile;

    /**
     * @param sourceFile the source file to return.
     */
    public BasicSourceFileProviderTask(final SourceFile sourceFile) {
      this.sourceFile = sourceFile;
    }

    public SourceFile getSourceFile() {
      return sourceFile;
    }

    public void execute(final Map<Object, Object> context) throws Exception {
      // nothing to do
    }

    public Object getResult() {
      return getSourceFile();
    }
  }

  // --------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // --------------------------------------------------------------------------

  public String[] listFc() {
    return new String[]{BUILDER_ITF};
  }

  public void bindFc(final String s, final Object o)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException {

    if (s == null) {
      throw new IllegalArgumentException("Interface name can't be null");
    }

    if (s.equals(BUILDER_ITF)) {
      this.builderItf = (SourceFileBuilder) o;
    } else {
      throw new NoSuchInterfaceException("There is no interface named '" + s
          + "'");
    }
  }

  public Object lookupFc(final String s) throws NoSuchInterfaceException {
    if (s == null) {
      throw new IllegalArgumentException("Interface name can't be null");
    }

    if (s.equals(BUILDER_ITF)) {
      return this.builderItf;
    } else {
      throw new NoSuchInterfaceException("There is no interface named '" + s
          + "'");
    }
  }

  public void unbindFc(final String s) throws NoSuchInterfaceException,
      IllegalBindingException, IllegalLifeCycleException {

    if (s == null) {
      throw new IllegalArgumentException("Interface name can't be null");
    }

    if (s.equals(BUILDER_ITF)) {
      this.builderItf = null;
    } else {
      throw new NoSuchInterfaceException("There is no interface named '" + s
          + "'");
    }
  }
}
